<?php

namespace Drupal\page_speed_for_seo\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;


class SeltechPageSpeedForm extends FormBase
{

  public function getFormId()
  {
    return "page_speed_for_seo_form";
  }

  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $form['site_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link target'),
      '#description' => $this->t('Please add your url like: https://seltech.io/'),
      '#required' => true,
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email'),
      '#description' => $this->t('Please add your email address.'),
      '#required' => true,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'Submit',
    ];

    $result = $form_state->getValue('result');
    if ($result) {
      $form['result'] = $result;
    }

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state)
  {

  }

  public function submitForm(array &$form, FormStateInterface $form_state)
  {

    $url = $form_state->getValues()["site_url"];
    $email = $form_state->getValues()["email"];
    $api_key = 'AIzaSyBhN_qfA3rxvkPsNdtSkWd9gJ8I9yAosws';

    // Send request to Page Speed API.
    $page_speed_result = json_decode($this->_page_speed_for_seo_requesto_to_api($api_key, $url), true)["lighthouseResult"]["categories"];

    // Get all values from Page speed API.
    $page_speed_values = [];

    if (!empty($page_speed_result)) {
      foreach ($page_speed_result as $key => $category) {
        $page_speed_values[$key] = $category["score"] * 100;
      }

      // Save the inputs in database and send email to the user.
      $this->_page_speed_for_seo_send_email($page_speed_values, $email, $url);
      $this->_page_speed_for_seo_save_data($email, $url);

    } else {
      $page_speed_values = [
        'No results found. Please check if you entered the url correctly and try again.' => 0
      ];
    }

    // Result for page overview.
    $result = [
      '#theme' => 'page_speed_for_seo',
      '#page_speed_result' => $page_speed_values,
    ];

    $form_state->setRebuild();
    $form_state->setValue('result', $result);
  }

  public function _page_speed_for_seo_requesto_to_api($api_key, $url): bool|string
  {
    $url_sh = "https://www.googleapis.com/pagespeedonline/v5/runPagespeed?url=$url&fields=lighthouseResult%2Fcategories%2F*%2Fscore&prettyPrint=false&strategy=desktop&category=performance&category=pwa&category=best-practices&category=accessibility&category=seo&key=$api_key";
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_URL, $url_sh);
    $result = curl_exec($ch);
    curl_close($ch);

    return $result;
  }

  public function _page_speed_for_seo_send_email($page_speed_values, $email, $url)
  {

    // Result for emails overview.
    $result = [
      '#theme' => 'page_speed_for_seo_email',
      '#page_speed_result' => $page_speed_values,
      '#email' => $email,
      '#link_target' => $url,

    ];

    // Send emails.
    $emailFactory = \Drupal::service('email_factory');
    return $emailFactory->newTypedEmail('page_speed_for_seo', 'pagespeed_mail_key')
      ->setSubject('SELTech checking results')
      ->setTo([$email])
      ->setBody(['#markup' => \Drupal::service('renderer')->renderPlain($result)])
      ->send();
  }

  public function _page_speed_for_seo_save_data($email, $url): int|string|null
  {
    // Save submissions in database.
    return \Drupal::database()->insert('page_speed_for_seo_submissions')
      ->fields([
        'email' => $email,
        'url' => $url,
      ])
      ->execute();
  }
}
